
// ************************************************************************
// Infrared-Routines                                            VERSION 1.1
// uses ext.Int0 (PORTD.2) and Timer0 (Osz. 8MHz)
                                              
// IR_Code      :(int) Highbyte = adress, lowbyte = command  
//                                          
// New_IR_Code : new IR-Code received (has to be cleared by polling!)
// ************************************************************************

#include <90s4434.h> 

char           New_IR_Code; // Globale Variable. Dieses Bit wird nach Empfang eines Zeichens gesetzt
unsigned int  IR_Code;             // Globale Variable. Enthaelt das empfangene Zeichen
bit            Decodierung_Lauft;   // ist gesetzt, waehrend Decodierung

// ************************************************************************
// Init IR
// ************************************************************************
void InitIR(void)
{

  GIMSK |= 0x40;           // ext. Int0 enable   
  MCUCR &= ~0x03;          // Trigger 
  MCUCR |= 0x02;           // on falling edge 
  
  GIFR   = 0x40;           // clear Int0-Flag

  TCCR0 = 0x04;           // Timer0 auf 31,250kHz ( bei 8Mhz Osz.)
  Decodierung_Lauft = 0;
  New_IR_Code = 0; 
  IR_Code=0;    
  
  #asm("sei")             // interrupts global freigeben

}
// **************************************************************************
// * aufgerufen durch: externer Interrupt
// **************************************************************************
// * Die Funktion wird durch einen externen Interrupt aufgerufen.
// * Sie dient nur zum Starten der Abtastsequenz und zum Triggern auf die
// * Signalflanken in der Mitte der Daten.
// **************************************************************************
interrupt [EXT_INT0] void ext_int0_isr(void)
{
    // Zeitbasis fuer Timer einstellen
    // Entspricht 1,5 mal einer Bitlaenge
 
     TCNT0 = -41;              
     TIMSK |= 0x01;                         // Timer 0 starten, 
     if(Decodierung_Lauft) GIMSK &= ~0x40; // die erneute Int.-Auslsung soll
		                                   // nur die Timer-Routine erlauben
}

// **************************************************************************
// * aufgerufen durch: Timerueberlauf Timer 0
// **************************************************************************
// * Die Funktion wird durch einen Timer0-Interrupt aufgerufen.
// * Der Timer wird vom ext. Int0 gestartet.
// * Das Infrarotsignal wird hier abgetastet und die Wertigkeiten der
// * Bits entsprechend aufaddiert. Das empfangende Zeichen wird in der
// * globalen Variablen IR_Code gespeichert. Nach Empfang wird das Bit
// * Neues_Zeichen gesetzt.
// **************************************************************************
interrupt [TIM0_OVF] void timer0_ovf_isr(void)
{
static unsigned int  Shift,IR_Code_tmp;
static unsigned char IR_Zaehler=0,IRSperrCounter=0;
// Reinitialize Timer's 0 value
TCNT0 = -41;
                                            
 if (IRSperrCounter) 
 { IRSperrCounter--;
   if (!IRSperrCounter) 
   { TIMSK  &= ~0x01;   // Timer0 sperre wird durch ExtInt0 gestartet
     GIFR   &= ~0x40;   // IntFlag Loeschen
     GIMSK  |=  0x40;   // externen Interrupt wieder freigenben
   }  
 }
 else
 {
  if(IR_Zaehler > 1)
	{
	  Decodierung_Lauft = 1;
	    {
         GIFR &= ~0x40;            // Interrupt Flag loeschen
	     if(IR_PP) MCUCR &= ~0x01; // naechste fallende  Flanke soll Int0 ausloesen
	     else      MCUCR |= 1;    // naechste steigende Flanke soll Int0 ausloesen
	     GIMSK |= 0x40;            // externen Int frei
   
	    }
	}
 
  if(!IR_Zaehler)              // es beginnt eine neue Abtastung
  {                           // es handelt sich um das 1. Sync-Bit
   IR_Code_tmp=0;             // Merker lschen
   Shift=0x8000; //;8192;                // Maske auf MSB 
  }
  else
  if(IR_PP==1)                // das empfangende Bit ist logisch High
  {  if(IR_Zaehler>2)         // (IR_Zaehler ist gleich 1 beim 2. Sync-Bit)
      {                       // 2 --> Toggle-Bit (wird bersprungen)
        IR_Code_tmp |= Shift; // entsprechendse Bit setzen 
      }
  }
   
  Shift >>=1;                   // Shiftregister um 1 nach rechts schieben
                               // (fr das nchste empf. Bit  
  if (Shift == 128) Shift = 32; // Adresse ins HighByte, Kommando ins LowByte                            
 
  if(IR_Zaehler++==15)        // Das IR-Wort wurde vollstaendig abgetastet
    {
     
       GIFR   &= ~0x40;      // IntFlag Loeschen
       MCUCR  &= ~0x01;      // fallende Flanke
       IR_Zaehler = 0;
       Decodierung_Lauft=0;  
       if ((HIGH(IR_Code_tmp)==ALLOWED1)|| (HIGH(IR_Code_tmp)==ALLOWED2)||
            (HIGH(IR_Code_tmp)==ALLOWED3)||(HIGH(IR_Code_tmp)==ALLOWED4)||
            (HIGH(IR_Code_tmp)==ALLOWED5) )
       { // nur erlaubt Adressen werden akzepetiert
         IR_Code=IR_Code_tmp;  // IR-Wort in die globale Variable schreiben
         New_IR_Code=1;        // Es ist ein neues Zeichen eingetragen worden
         IRSperrCounter = 200; // weitere Kommandos fr 200 * 1,3ms unterdrcken 
       } 
       else 
         GIMSK  |= 0x40;     // externen Interrupt wieder freigenben
     
     }
 }   
}                      

